@@ -10,16 +10,24 @@ import re
 import sys
 import types
 
-from typing import Any, Dict  # NOQA
-
-from ruamel.yaml.error import *                               # NOQA
+from ruamel.yaml.error import (MarkedYAMLError)
 from ruamel.yaml.nodes import *                               # NOQA
-from ruamel.yaml.compat import (utf8, builtins_module, to_str, PY2, PY3,
-                                ordereddict, text_type)
+from ruamel.yaml.nodes import (SequenceNode, MappingNode, ScalarNode)
+from ruamel.yaml.compat import (utf8, builtins_module, to_str, PY2, PY3,  # NOQA
+                                ordereddict, text_type, nprint)
 from ruamel.yaml.comments import *                               # NOQA
+from ruamel.yaml.comments import (CommentedMap, CommentedOrderedMap, CommentedSet,
+                                  CommentedKeySeq, CommentedSeq)
 from ruamel.yaml.scalarstring import *                           # NOQA
+from ruamel.yaml.scalarstring import (PreservedScalarString, SingleQuotedScalarString,
+                                      DoubleQuotedScalarString, ScalarString)
+from ruamel.yaml.scalarint import ScalarInt, BinaryInt, OctalInt, HexInt, HexCapsInt
 from ruamel.yaml.timestamp import TimeStamp
 
+if sys.version_info >= (3, 5, 2):
+    from typing import Any, Dict, List, Set, Generator  # NOQA
+
+
 __all__ = ['BaseConstructor', 'SafeConstructor', 'Constructor',
            'ConstructorError', 'RoundTripConstructor']
 
@@ -33,32 +41,57 @@ class BaseConstructor(object):
     yaml_constructors = {}   # type: Dict[Any, Any]
     yaml_multi_constructors = {}   # type: Dict[Any, Any]
 
-    def __init__(self, preserve_quotes=None):
-        self.constructed_objects = {}
-        self.recursive_objects = {}
-        self.state_generators = []
+    def __init__(self, preserve_quotes=None, loader=None):
+        # type: (bool, Any) -> None
+        self.loader = loader
+        if self.loader is not None:
+            self.loader._constructor = self
+        self.loader = loader
+        self.constructed_objects = {}  # type: Dict[Any, Any]
+        self.recursive_objects = {}  # type: Dict[Any, Any]
+        self.state_generators = []  # type: List[Any]
         self.deep_construct = False
         self._preserve_quotes = preserve_quotes
 
+    @property
+    def composer(self):
+        # type: () -> Any
+        try:
+            return self.loader._composer
+        except AttributeError:
+            print('slt', type(self))
+            print('slc', self.loader._composer)
+            print(dir(self))
+            raise
+
+    @property
+    def resolver(self):
+        # type: () -> Any
+        return self.loader._resolver
+
     def check_data(self):
+        # type: () -> Any
         # If there are more documents available?
-        return self.check_node()
+        return self.composer.check_node()
 
     def get_data(self):
+        # type: () -> Any
         # Construct and return the next document.
-        if self.check_node():
-            return self.construct_document(self.get_node())
+        if self.composer.check_node():
+            return self.construct_document(self.composer.get_node())
 
     def get_single_data(self):
+        # type: () -> Any
         # Ensure that the stream contains a single document and construct it.
-        node = self.get_single_node()
+        node = self.composer.get_single_node()
         if node is not None:
             return self.construct_document(node)
         return None
 
     def construct_document(self, node):
+        # type: (Any) -> Any
         data = self.construct_object(node)
-        while self.state_generators:
+        while bool(self.state_generators):
             state_generators = self.state_generators
             self.state_generators = []
             for generator in state_generators:
@@ -70,6 +103,7 @@ class BaseConstructor(object):
         return data
 
     def construct_object(self, node, deep=False):
+        # type: (Any, bool) -> Any
         """deep is True when creating an object/mapping recursively,
         in that case want the underlying elements available during construction
         """
@@ -100,20 +134,20 @@ class BaseConstructor(object):
                 elif None in self.yaml_constructors:
                     constructor = self.yaml_constructors[None]
                 elif isinstance(node, ScalarNode):
-                    constructor = self.__class__.construct_scalar
+                    constructor = self.__class__.construct_scalar  # type: ignore
                 elif isinstance(node, SequenceNode):
-                    constructor = self.__class__.construct_sequence
+                    constructor = self.__class__.construct_sequence  # type: ignore
                 elif isinstance(node, MappingNode):
-                    constructor = self.__class__.construct_mapping
+                    constructor = self.__class__.construct_mapping   # type: ignore
         if tag_suffix is None:
             data = constructor(self, node)
         else:
             data = constructor(self, tag_suffix, node)
         if isinstance(data, types.GeneratorType):
             generator = data
-            data = next(generator)
+            data = next(generator)  # type: ignore
             if self.deep_construct:
-                for dummy in generator:
+                for dummy in generator:  # type: ignore
                     pass
             else:
                 self.state_generators.append(generator)
@@ -124,6 +158,7 @@ class BaseConstructor(object):
         return data
 
     def construct_scalar(self, node):
+        # type: (Any) -> Any
         if not isinstance(node, ScalarNode):
             raise ConstructorError(
                 None, None,
@@ -132,6 +167,7 @@ class BaseConstructor(object):
         return node.value
 
     def construct_sequence(self, node, deep=False):
+        # type: (Any, bool) -> Any
         """deep is True when creating an object/mapping recursively,
         in that case want the underlying elements available during construction
         """
@@ -144,6 +180,7 @@ class BaseConstructor(object):
                 for child in node.value]
 
     def construct_mapping(self, node, deep=False):
+        # type: (Any, bool) -> Any
         """deep is True when creating an object/mapping recursively,
         in that case want the underlying elements available during construction
         """
@@ -157,7 +194,7 @@ class BaseConstructor(object):
             # keys can be list -> deep
             key = self.construct_object(key_node, deep=True)
             # lists are not hashable, but tuples are
-            if not isinstance(key, collections.Hashable):
+            if not isinstance(key, collections.Hashable):  # type: ignore
                 if isinstance(key, list):
                     key = tuple(key)
             if PY2:
@@ -179,6 +216,7 @@ class BaseConstructor(object):
         return mapping
 
     def construct_pairs(self, node, deep=False):
+        # type: (Any, bool) -> Any
         if not isinstance(node, MappingNode):
             raise ConstructorError(
                 None, None,
@@ -208,6 +246,7 @@ class BaseConstructor(object):
 
 class SafeConstructor(BaseConstructor):
     def construct_scalar(self, node):
+        # type: (Any) -> Any
         if isinstance(node, MappingNode):
             for key_node, value_node in node.value:
                 if key_node.tag == u'tag:yaml.org,2002:value':
@@ -215,12 +254,13 @@ class SafeConstructor(BaseConstructor):
         return BaseConstructor.construct_scalar(self, node)
 
     def flatten_mapping(self, node):
+        # type: (Any) -> Any
         """
         This implements the merge key feature http://yaml.org/type/merge.html
         by inserting keys from the merge dict/list of dicts if not yet
         available in this node
         """
-        merge = []
+        merge = []  # type: List[Any]
         index = 0
         while index < len(node.value):
             key_node, value_node = node.value[index]
@@ -254,10 +294,11 @@ class SafeConstructor(BaseConstructor):
                 index += 1
             else:
                 index += 1
-        if merge:
+        if bool(merge):
             node.value = merge + node.value
 
     def construct_mapping(self, node, deep=False):
+        # type: (Any, bool) -> Any
         """deep is True when creating an object/mapping recursively,
         in that case want the underlying elements available during construction
         """
@@ -266,6 +307,7 @@ class SafeConstructor(BaseConstructor):
         return BaseConstructor.construct_mapping(self, node, deep=deep)
 
     def construct_yaml_null(self, node):
+        # type: (Any) -> Any
         self.construct_scalar(node)
         return None
 
@@ -280,29 +322,31 @@ class SafeConstructor(BaseConstructor):
     }
 
     def construct_yaml_bool(self, node):
+        # type: (Any) -> bool
         value = self.construct_scalar(node)
         return self.bool_values[value.lower()]
 
     def construct_yaml_int(self, node):
-        value = to_str(self.construct_scalar(node))
-        value = value.replace('_', '')
+        # type: (Any) -> int
+        value_s = to_str(self.construct_scalar(node))
+        value_s = value_s.replace('_', '')
         sign = +1
-        if value[0] == '-':
+        if value_s[0] == '-':
             sign = -1
-        if value[0] in '+-':
-            value = value[1:]
-        if value == '0':
+        if value_s[0] in '+-':
+            value_s = value_s[1:]
+        if value_s == '0':
             return 0
-        elif value.startswith('0b'):
-            return sign*int(value[2:], 2)
-        elif value.startswith('0x'):
-            return sign*int(value[2:], 16)
-        elif value.startswith('0o'):
-            return sign*int(value[2:], 8)
-        elif self.processing_version != (1, 2) and value[0] == '0':
-            return sign*int(value, 8)
-        elif self.processing_version != (1, 2) and ':' in value:
-            digits = [int(part) for part in value.split(':')]
+        elif value_s.startswith('0b'):
+            return sign*int(value_s[2:], 2)
+        elif value_s.startswith('0x'):
+            return sign*int(value_s[2:], 16)
+        elif value_s.startswith('0o'):
+            return sign*int(value_s[2:], 8)
+        elif self.resolver.processing_version != (1, 2) and value_s[0] == '0':
+            return sign*int(value_s, 8)
+        elif self.resolver.processing_version != (1, 2) and ':' in value_s:
+            digits = [int(part) for part in value_s.split(':')]
             digits.reverse()
             base = 1
             value = 0
@@ -311,7 +355,7 @@ class SafeConstructor(BaseConstructor):
                 base *= 60
             return sign*value
         else:
-            return sign*int(value)
+            return sign*int(value_s)
 
     inf_value = 1e300
     while inf_value != inf_value*inf_value:
@@ -319,19 +363,20 @@ class SafeConstructor(BaseConstructor):
     nan_value = -inf_value/inf_value   # Trying to make a quiet NaN (like C99).
 
     def construct_yaml_float(self, node):
-        value = to_str(self.construct_scalar(node))
-        value = value.replace('_', '').lower()
+        # type: (Any) -> float
+        value_s = to_str(self.construct_scalar(node))
+        value_s = value_s.replace('_', '').lower()
         sign = +1
-        if value[0] == '-':
+        if value_s[0] == '-':
             sign = -1
-        if value[0] in '+-':
-            value = value[1:]
-        if value == '.inf':
+        if value_s[0] in '+-':
+            value_s = value_s[1:]
+        if value_s == '.inf':
             return sign*self.inf_value
-        elif value == '.nan':
+        elif value_s == '.nan':
             return self.nan_value
-        elif ':' in value:
-            digits = [float(part) for part in value.split(':')]
+        elif ':' in value_s:
+            digits = [float(part) for part in value_s.split(':')]
             digits.reverse()
             base = 1
             value = 0.0
@@ -340,10 +385,11 @@ class SafeConstructor(BaseConstructor):
                 base *= 60
             return sign*value
         else:
-            return sign*float(value)
+            return sign*float(value_s)
 
     if PY3:
         def construct_yaml_binary(self, node):
+            # type: (Any) -> Any
             try:
                 value = self.construct_scalar(node).encode('ascii')
             except UnicodeEncodeError as exc:
@@ -362,6 +408,7 @@ class SafeConstructor(BaseConstructor):
                     "failed to decode base64 data: %s" % exc, node.start_mark)
     else:
         def construct_yaml_binary(self, node):
+            # type: (Any) -> Any
             value = self.construct_scalar(node)
             try:
                 return to_str(value).decode('base64')
@@ -383,6 +430,7 @@ class SafeConstructor(BaseConstructor):
           (?::(?P<tz_minute>[0-9][0-9]))?))?)?$''', re.X)
 
     def construct_yaml_timestamp(self, node, values=None):
+        # type: (Any, Any) -> Any
         if values is None:
             match = self.timestamp_regexp.match(node.value)
             values = match.groupdict()
@@ -396,26 +444,28 @@ class SafeConstructor(BaseConstructor):
         second = int(values['second'])
         fraction = 0
         if values['fraction']:
-            fraction = values['fraction'][:6]
-            while len(fraction) < 6:
-                fraction += '0'
-            fraction = int(fraction)
+            fraction_s = values['fraction'][:6]
+            while len(fraction_s) < 6:
+                fraction_s += '0'
+            fraction = int(fraction_s)
             if len(values['fraction']) > 6 and int(values['fraction'][6]) > 4:
                 fraction += 1
         delta = None
         if values['tz_sign']:
             tz_hour = int(values['tz_hour'])
-            tz_minute = int(values['tz_minute'] or 0)
+            minutes = values['tz_minute']
+            tz_minute = int(minutes) if minutes else 0
             delta = datetime.timedelta(hours=tz_hour, minutes=tz_minute)
             if values['tz_sign'] == '-':
                 delta = -delta
         data = datetime.datetime(year, month, day, hour, minute, second,
                                  fraction)
-        if delta:
+        if delta:  # type: ignore
             data -= delta
         return data
 
     def construct_yaml_omap(self, node):
+        # type: (Any) -> Any
         # Note: we do now check for duplicate keys
         omap = ordereddict()
         yield omap
@@ -443,8 +493,9 @@ class SafeConstructor(BaseConstructor):
             omap[key] = value
 
     def construct_yaml_pairs(self, node):
+        # type: (Any) -> Any
         # Note: the same code as `construct_yaml_omap`.
-        pairs = []
+        pairs = []  # type: List[Any]
         yield pairs
         if not isinstance(node, SequenceNode):
             raise ConstructorError(
@@ -469,12 +520,14 @@ class SafeConstructor(BaseConstructor):
             pairs.append((key, value))
 
     def construct_yaml_set(self, node):
-        data = set()
+        # type: (Any) -> Any
+        data = set()    # type: Set[Any]
         yield data
         value = self.construct_mapping(node)
         data.update(value)
 
     def construct_yaml_str(self, node):
+        # type: (Any) -> Any
         value = self.construct_scalar(node)
         if PY3:
             return value
@@ -484,17 +537,20 @@ class SafeConstructor(BaseConstructor):
             return value
 
     def construct_yaml_seq(self, node):
-        data = []
+        # type: (Any) -> Any
+        data = []  # type: List[Any]
         yield data
         data.extend(self.construct_sequence(node))
 
     def construct_yaml_map(self, node):
-        data = {}
+        # type: (Any) -> Any
+        data = {}  # type: Dict[Any, Any]
         yield data
         value = self.construct_mapping(node)
         data.update(value)
 
     def construct_yaml_object(self, node, cls):
+        # type: (Any, Any) -> Any
         data = cls.__new__(cls)
         yield data
         if hasattr(data, '__setstate__'):
@@ -505,6 +561,7 @@ class SafeConstructor(BaseConstructor):
             data.__dict__.update(state)
 
     def construct_undefined(self, node):
+        # type: (Any) -> None
         raise ConstructorError(
             None, None,
             "could not determine a constructor for the tag %r" %
@@ -562,17 +619,24 @@ SafeConstructor.add_constructor(
 SafeConstructor.add_constructor(
     None, SafeConstructor.construct_undefined)
 
+if PY2:
+    class classobj:
+        pass
+
 
 class Constructor(SafeConstructor):
 
     def construct_python_str(self, node):
+        # type: (Any) -> Any
         return utf8(self.construct_scalar(node))
 
     def construct_python_unicode(self, node):
+        # type: (Any) -> Any
         return self.construct_scalar(node)
 
     if PY3:
         def construct_python_bytes(self, node):
+            # type: (Any) -> Any
             try:
                 value = self.construct_scalar(node).encode('ascii')
             except UnicodeEncodeError as exc:
@@ -591,18 +655,22 @@ class Constructor(SafeConstructor):
                     "failed to decode base64 data: %s" % exc, node.start_mark)
 
     def construct_python_long(self, node):
+        # type: (Any) -> int
         val = self.construct_yaml_int(node)
         if PY3:
             return val
         return int(val)
 
     def construct_python_complex(self, node):
+        # type: (Any) -> Any
         return complex(self.construct_scalar(node))
 
     def construct_python_tuple(self, node):
+        # type: (Any) -> Any
         return tuple(self.construct_sequence(node))
 
     def find_python_module(self, name, mark):
+        # type: (Any, Any) -> Any
         if not name:
             raise ConstructorError(
                 "while constructing a Python module", mark,
@@ -616,6 +684,7 @@ class Constructor(SafeConstructor):
         return sys.modules[name]
 
     def find_python_name(self, name, mark):
+        # type: (Any, Any) -> Any
         if not name:
             raise ConstructorError(
                 "while constructing a Python object", mark,
@@ -640,6 +709,7 @@ class Constructor(SafeConstructor):
         return getattr(module, object_name)
 
     def construct_python_name(self, suffix, node):
+        # type: (Any, Any) -> Any
         value = self.construct_scalar(node)
         if value:
             raise ConstructorError(
@@ -649,6 +719,7 @@ class Constructor(SafeConstructor):
         return self.find_python_name(suffix, node.start_mark)
 
     def construct_python_module(self, suffix, node):
+        # type: (Any, Any) -> Any
         value = self.construct_scalar(node)
         if value:
             raise ConstructorError(
@@ -657,12 +728,9 @@ class Constructor(SafeConstructor):
                 node.start_mark)
         return self.find_python_module(suffix, node.start_mark)
 
-    if PY2:
-        class classobj:
-            pass
-
     def make_python_instance(self, suffix, node,
                              args=None, kwds=None, newobj=False):
+        # type: (Any, Any, Any, Any, bool) -> Any
         if not args:
             args = []
         if not kwds:
@@ -674,9 +742,9 @@ class Constructor(SafeConstructor):
             else:
                 return cls(*args, **kwds)
         else:
-            if newobj and isinstance(cls, type(self.classobj))  \
+            if newobj and isinstance(cls, type(classobj))  \
                     and not args and not kwds:
-                instance = self.classobj()
+                instance = classobj()
                 instance.__class__ = cls
                 return instance
             elif newobj and isinstance(cls, type):
@@ -685,10 +753,11 @@ class Constructor(SafeConstructor):
                 return cls(*args, **kwds)
 
     def set_python_instance_state(self, instance, state):
+        # type: (Any, Any) -> None
         if hasattr(instance, '__setstate__'):
             instance.__setstate__(state)
         else:
-            slotstate = {}
+            slotstate = {}  # type: Dict[Any, Any]
             if isinstance(state, tuple) and len(state) == 2:
                 state, slotstate = state
             if hasattr(instance, '__dict__'):
@@ -699,6 +768,7 @@ class Constructor(SafeConstructor):
                 setattr(object, key, value)
 
     def construct_python_object(self, suffix, node):
+        # type: (Any, Any) -> Any
         # Format:
         #   !!python/object:module.name { ... state ... }
         instance = self.make_python_instance(suffix, node, newobj=True)
@@ -708,6 +778,7 @@ class Constructor(SafeConstructor):
         self.set_python_instance_state(instance, state)
 
     def construct_python_object_apply(self, suffix, node, newobj=False):
+        # type: (Any, Any, bool) -> Any
         # Format:
         #   !!python/object/apply       # (or !!python/object/new)
         #   args: [ ... arguments ... ]
@@ -721,10 +792,10 @@ class Constructor(SafeConstructor):
         # is how an object is created, check make_python_instance for details.
         if isinstance(node, SequenceNode):
             args = self.construct_sequence(node, deep=True)
-            kwds = {}
-            state = {}
-            listitems = []
-            dictitems = {}
+            kwds = {}  # type: Dict[Any, Any]
+            state = {}  # type: Dict[Any, Any]
+            listitems = []  # type: List[Any]
+            dictitems = {}  # type: Dict[Any, Any]
         else:
             value = self.construct_mapping(node, deep=True)
             args = value.get('args', [])
@@ -733,16 +804,17 @@ class Constructor(SafeConstructor):
             listitems = value.get('listitems', [])
             dictitems = value.get('dictitems', {})
         instance = self.make_python_instance(suffix, node, args, kwds, newobj)
-        if state:
+        if bool(state):
             self.set_python_instance_state(instance, state)
-        if listitems:
+        if bool(listitems):
             instance.extend(listitems)
-        if dictitems:
+        if bool(dictitems):
             for key in dictitems:
                 instance[key] = dictitems[key]
         return instance
 
     def construct_python_object_new(self, suffix, node):
+        # type: (Any, Any) -> Any
         return self.construct_python_object_apply(suffix, node, newobj=True)
 
 Constructor.add_constructor(
@@ -821,6 +893,7 @@ class RoundTripConstructor(SafeConstructor):
     """
 
     def construct_scalar(self, node):
+        # type: (Any) -> Any
         if not isinstance(node, ScalarNode):
             raise ConstructorError(
                 None, None,
@@ -829,14 +902,91 @@ class RoundTripConstructor(SafeConstructor):
 
         if node.style == '|' and isinstance(node.value, text_type):
             return PreservedScalarString(node.value)
-        elif self._preserve_quotes and isinstance(node.value, text_type):
+        elif bool(self._preserve_quotes) and isinstance(node.value, text_type):
             if node.style == "'":
                 return SingleQuotedScalarString(node.value)
             if node.style == '"':
                 return DoubleQuotedScalarString(node.value)
         return node.value
 
+    def construct_yaml_int(self, node):
+        # type: (Any) -> Any
+        width = None  # type: Any
+        value_su = to_str(self.construct_scalar(node))
+        try:
+            sx = value_su.rstrip('_')
+            underscore = [len(sx) - sx.rindex('_') - 1, False, False]  # type: Any
+        except ValueError:
+            underscore = None
+        except IndexError:
+            underscore = None
+        value_s = value_su.replace('_', '')
+        sign = +1
+        if value_s[0] == '-':
+            sign = -1
+        if value_s[0] in '+-':
+            value_s = value_s[1:]
+        if value_s == '0':
+            return 0
+        elif value_s.startswith('0b'):
+            if self.resolver.processing_version > (1, 1) and value_s[2] == '0':
+                width = len(value_s[2:])
+            if underscore is not None:
+                underscore[1] = value_su[2] == '_'
+                underscore[2] = len(value_su[2:]) > 1 and value_su[-1] == '_'
+            return BinaryInt(sign*int(value_s[2:], 2), width=width,   # type: ignore
+                             underscore=underscore)
+        elif value_s.startswith('0x'):
+            # default to lower-case if no a-fA-F in string
+            if self.resolver.processing_version > (1, 1) and value_s[2] == '0':
+                width = len(value_s[2:])
+            hex_fun = HexInt  # type: Any
+            for ch in value_s[2:]:
+                if ch in 'ABCDEF':  # first non-digit is capital
+                    hex_fun = HexCapsInt
+                    break
+                if ch in 'abcdef':
+                    break
+            if underscore is not None:
+                underscore[1] = value_su[2] == '_'
+                underscore[2] = len(value_su[2:]) > 1 and value_su[-1] == '_'
+            return hex_fun(sign*int(value_s[2:], 16), width=width, underscore=underscore)
+        elif value_s.startswith('0o'):
+            if self.resolver.processing_version > (1, 1) and value_s[2] == '0':
+                width = len(value_s[2:])
+            if underscore is not None:
+                underscore[1] = value_su[2] == '_'
+                underscore[2] = len(value_su[2:]) > 1 and value_su[-1] == '_'
+            return OctalInt(sign*int(value_s[2:], 8), width=width,  # type: ignore
+                            underscore=underscore)
+        elif self.resolver.processing_version != (1, 2) and value_s[0] == '0':
+            return sign*int(value_s, 8)
+        elif self.resolver.processing_version != (1, 2) and ':' in value_s:
+            digits = [int(part) for part in value_s.split(':')]
+            digits.reverse()
+            base = 1
+            value = 0
+            for digit in digits:
+                value += digit*base
+                base *= 60
+            return sign*value
+        elif self.resolver.processing_version > (1, 1) and value_s[0] == '0':
+            # not an octal, an integer with leading zero(s)
+            if underscore is not None:
+                # cannot have a leading underscore
+                underscore[2] = len(value_su) > 1 and value_su[-1] == '_'
+            return ScalarInt(sign*int(value_s), width=len(value_s),  # type: ignore
+                             underscore=underscore)
+        elif underscore:
+            # cannot have a leading underscore
+            underscore[2] = len(value_su) > 1 and value_su[-1] == '_'
+            return ScalarInt(sign*int(value_s), width=None,  # type: ignore
+                             underscore=underscore)
+        else:
+            return sign*int(value_s)
+
     def construct_yaml_str(self, node):
+        # type: (Any) -> Any
         value = self.construct_scalar(node)
         if isinstance(value, ScalarString):
             return value
@@ -850,7 +1000,8 @@ class RoundTripConstructor(SafeConstructor):
         except UnicodeEncodeError:
             return value
 
-    def construct_sequence(self, node, seqtyp, deep=False):
+    def construct_rt_sequence(self, node, seqtyp, deep=False):
+        # type: (Any, Any, bool) -> Any
         if not isinstance(node, SequenceNode):
             raise ConstructorError(
                 None, None,
@@ -861,7 +1012,7 @@ class RoundTripConstructor(SafeConstructor):
             seqtyp._yaml_add_comment(node.comment[:2])
             if len(node.comment) > 2:
                 seqtyp.yaml_end_comment_extend(node.comment[2], clear=True)
-        if node.anchor:
+        if node.anchor:  # type: ignore
             from ruamel.yaml.serializer import templated_id
             if not templated_id(node.anchor):
                 seqtyp.yaml_set_anchor(node.anchor)
@@ -874,6 +1025,7 @@ class RoundTripConstructor(SafeConstructor):
         return ret_val
 
     def flatten_mapping(self, node):
+        # type: (Any) -> Any
         """
         This implements the merge key feature http://yaml.org/type/merge.html
         by inserting keys from the merge dict/list of dicts if not yet
@@ -881,6 +1033,7 @@ class RoundTripConstructor(SafeConstructor):
         """
 
         def constructed(value_node):
+            # type: (Any) -> Any
             # If the contents of a merge are defined within the
             # merge marker, then they won't have been constructed
             # yet. But if they were already constructed, we need to use
@@ -935,9 +1088,11 @@ class RoundTripConstructor(SafeConstructor):
         #     node.value = merge + node.value
 
     def _sentinel(self):
+        # type: () -> None
         pass
 
-    def construct_mapping(self, node, maptyp, deep=False):
+    def construct_mapping(self, node, maptyp=None, deep=False):  # type: ignore
+        # type: (Any, Any, bool) -> Any
         if not isinstance(node, MappingNode):
             raise ConstructorError(
                 None, None,
@@ -949,7 +1104,7 @@ class RoundTripConstructor(SafeConstructor):
             maptyp._yaml_add_comment(node.comment[:2])
             if len(node.comment) > 2:
                 maptyp.yaml_end_comment_extend(node.comment[2], clear=True)
-        if node.anchor:
+        if node.anchor:  # type: ignore
             from ruamel.yaml.serializer import templated_id
             if not templated_id(node.anchor):
                 maptyp.yaml_set_anchor(node.anchor)
@@ -958,13 +1113,14 @@ class RoundTripConstructor(SafeConstructor):
             # keys can be list -> deep
             key = self.construct_object(key_node, deep=True)
             # lists are not hashable, but tuples are
-            if not isinstance(key, collections.Hashable):
+            if not isinstance(key, collections.Hashable):  # type: ignore
                 if isinstance(key, list):
-                    key = CommentedKeySeq(key)
+                    key_a = CommentedKeySeq(key)
                     if key_node.flow_style is True:
-                        key.fa.set_flow_style()
+                        key_a.fa.set_flow_style()
                     elif key_node.flow_style is False:
-                        key.fa.set_block_style()
+                        key_a.fa.set_block_style()
+                    key = key_a
                     # key = tuple(key)
             if PY2:
                 try:
@@ -1004,6 +1160,7 @@ class RoundTripConstructor(SafeConstructor):
             maptyp.add_yaml_merge(merge_map)
 
     def construct_setting(self, node, typ, deep=False):
+        # type: (Any, Any, bool) -> Any
         if not isinstance(node, MappingNode):
             raise ConstructorError(
                 None, None,
@@ -1013,7 +1170,7 @@ class RoundTripConstructor(SafeConstructor):
             typ._yaml_add_comment(node.comment[:2])
             if len(node.comment) > 2:
                 typ.yaml_end_comment_extend(node.comment[2], clear=True)
-        if node.anchor:
+        if node.anchor:  # type: ignore
             from ruamel.yaml.serializer import templated_id
             if not templated_id(node.anchor):
                 typ.yaml_set_anchor(node.anchor)
@@ -1021,7 +1178,7 @@ class RoundTripConstructor(SafeConstructor):
             # keys can be list -> deep
             key = self.construct_object(key_node, deep=True)
             # lists are not hashable, but tuples are
-            if not isinstance(key, collections.Hashable):
+            if not isinstance(key, collections.Hashable):  # type: ignore
                 if isinstance(key, list):
                     key = tuple(key)
             if PY2:
@@ -1045,6 +1202,7 @@ class RoundTripConstructor(SafeConstructor):
             typ.add(key)
 
     def construct_yaml_seq(self, node):
+        # type: (Any) -> Any
         data = CommentedSeq()
         data._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
         if node.flow_style is True:
@@ -1054,9 +1212,10 @@ class RoundTripConstructor(SafeConstructor):
         if node.comment:
             data._yaml_add_comment(node.comment)
         yield data
-        data.extend(self.construct_sequence(node, data))
+        data.extend(self.construct_rt_sequence(node, data))
 
     def construct_yaml_map(self, node):
+        # type: (Any) -> Any
         data = CommentedMap()
         data._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
         if node.flow_style is True:
@@ -1067,6 +1226,7 @@ class RoundTripConstructor(SafeConstructor):
         self.construct_mapping(node, data)
 
     def construct_yaml_omap(self, node):
+        # type: (Any) -> Any
         # Note: we do now check for duplicate keys
         omap = CommentedOrderedMap()
         omap._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
@@ -1109,12 +1269,14 @@ class RoundTripConstructor(SafeConstructor):
             omap[key] = value
 
     def construct_yaml_set(self, node):
+        # type: (Any) -> Any
         data = CommentedSet()
         data._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
         yield data
         self.construct_setting(node, data)
 
     def construct_undefined(self, node):
+        # type: (Any) -> Any
         try:
             data = CommentedMap()
             data._yaml_set_line_col(node.start_mark.line, node.start_mark.column)
@@ -1132,7 +1294,8 @@ class RoundTripConstructor(SafeConstructor):
                 utf8(node.tag),
                 node.start_mark)
 
-    def construct_yaml_timestamp(self, node):
+    def construct_yaml_timestamp(self, node, values=None):
+        # type: (Any, Any) -> Any
         match = self.timestamp_regexp.match(node.value)
         values = match.groupdict()
         if not values['hour']:
@@ -1150,20 +1313,21 @@ class RoundTripConstructor(SafeConstructor):
         second = int(values['second'])
         fraction = 0
         if values['fraction']:
-            fraction = values['fraction'][:6]
-            while len(fraction) < 6:
-                fraction += '0'
-            fraction = int(fraction)
+            fraction_s = values['fraction'][:6]
+            while len(fraction_s) < 6:
+                fraction_s += '0'
+            fraction = int(fraction_s)
             if len(values['fraction']) > 6 and int(values['fraction'][6]) > 4:
                 fraction += 1
         delta = None
         if values['tz_sign']:
             tz_hour = int(values['tz_hour'])
-            tz_minute = int(values['tz_minute'] or 0)
+            minutes = values['tz_minute']
+            tz_minute = int(minutes) if minutes else 0
             delta = datetime.timedelta(hours=tz_hour, minutes=tz_minute)
             if values['tz_sign'] == '-':
                 delta = -delta
-        if delta:
+        if delta:  # type: ignore
             dt = datetime.datetime(year, month, day, hour, minute)
             dt -= delta
             data = TimeStamp(dt.year, dt.month, dt.day, dt.hour, dt.minute,
